/*
 * Copyright 2002-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.web.servlet.view;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.hdiv.config.HDIVConfig;
import org.hdiv.util.HDIVUtil;
import org.hdiv.util.SpringRequestUtilsHDIV;
import org.springframework.web.servlet.view.RedirectView;

/**
 * <p>
 * View that redirects to an absolute, context relative, or current request
 * relative URL, exposing all model attributes as HTTP query parameters.
 * 
 * <p>
 * A URL for this view is supposed to be a HTTP redirect URL, i.e. suitable for
 * HttpServletResponse's <code>sendRedirect</code> method, which is what
 * actually does the redirect if the HTTP 1.0 flag is on, or via sending back an
 * HTTP 303 code - if the HTTP 1.0 compatibility flag is off.
 * 
 * <p>
 * Note that while the default value for the "contextRelative" flag is off, you
 * will probably want to almost always set it to true. With the flag off, URLs
 * starting with "/" are considered relative to the web server root, while with
 * the flag on, they are considered relative to the web application root. Since
 * most web apps will never know or care what their context path actually is,
 * they are much better off setting this flag to true, and submitting paths
 * which are to be considered relative to the web application root.
 * 
 * <p>
 * Note that in a Servlet 2.2 environment, i.e. a servlet container which is
 * only compliant to the limits of this spec, this class will probably fail when
 * feeding in URLs which are not fully absolute, or relative to the current
 * request (no leading "/"), as these are the only two types of URL that
 * <code>sendRedirect</code> supports in a Servlet 2.2 environment.
 * 
 * @author Gorka Vicente
 * @since HDIV 2.0.3
 * @see #setHttp10Compatible
 * @see javax.servlet.http.HttpServletResponse#sendRedirect
 */
public class RedirectViewHDIV extends RedirectView {

	/**
	 * Constructor for use as a bean.
	 */
	public RedirectViewHDIV() {
		super();
	}

	/**
	 * Create a new RedirectView with the given URL.
	 * <p>
	 * The given URL will be considered as relative to the web server, not as
	 * relative to the current ServletContext.
	 * 
	 * @param url the URL to redirect to
	 * @see #RedirectView(String, boolean)
	 */
	public RedirectViewHDIV(String url) {
		super(url);
	}

	/**
	 * Create a new RedirectView with the given URL.
	 * 
	 * @param url the URL to redirect to
	 * @param contextRelative whether to interpret the given URL as relative to
	 *            the current ServletContext
	 */
	public RedirectViewHDIV(String url, boolean contextRelative) {
		super(url, contextRelative);
	}

	/**
	 * Create a new RedirectView with the given URL.
	 * 
	 * @param url the URL to redirect to
	 * @param contextRelative whether to interpret the given URL as relative to
	 *            the current ServletContext
	 * @param http10Compatible whether to stay compatible with HTTP 1.0 clients
	 */
	public RedirectViewHDIV(String url, boolean contextRelative, boolean http10Compatible) {

		super(url, contextRelative, http10Compatible);
	}

	/**
	 * Send a redirect back to the HTTP client and adds HDIV state as a
	 * parameter if <code>targetUrl</code> references our application.
	 * 
	 * @param request current HTTP request (allows for reacting to request
	 *            method)
	 * @param response current HTTP response (for sending response headers)
	 * @param targetUrl the target URL to redirect to
	 * @param http10Compatible whether to stay compatible with HTTP 1.0 clients
	 * @throws IOException if thrown by response methods
	 */
	protected void sendRedirect(HttpServletRequest request, HttpServletResponse response, String targetUrl,
			boolean http10Compatible) throws IOException {

		HDIVConfig hdivConfig = (HDIVConfig) HDIVUtil.getApplication().getBean("config");

		if (SpringRequestUtilsHDIV.hasActionOrServletExtension(targetUrl, hdivConfig.getProtectedURLPatterns())) {
			targetUrl = SpringRequestUtilsHDIV.addHDIVParameterIfNecessary(request, targetUrl);
		}

		if (http10Compatible) {
			// Always send status code 302.
			response.sendRedirect(response.encodeRedirectURL(targetUrl));
		} else {
			// Correct HTTP status code is 303, in particular for POST requests.
			response.setStatus(303);
			response.setHeader("Location", response.encodeRedirectURL(targetUrl));
		}
	}

}
